// Example Java Code for the lecture on Graph Theory in DM534 by Daniel Merkle Nov. 8, 2018.
// For simplicity, no Exception Handling is implemented.

import java.util.Random;

public class Timing {

    // Assume M and N are both square (size x size) matrices 
    public static int[][] multiplyModSquared(int[][] M, int[][] N) {
	int inf = Integer.MAX_VALUE;
	
        int size = M.length;

        int[][] C = new int[size][size];
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
		C[i][j] = inf;
            }
        }

	int add;
        for (int i = 0; i < size; i++) { 
            for (int j = 0; j < size; j++) { 
                for (int k = 0; k < size; k++) {
		    if (M[i][k] == inf || N[k][j] == inf)
			C[i][j] = Math.min(C[i][j], inf);
		    else
			C[i][j] = Math.min(C[i][j], M[i][k] + N[k][j]);
                }
            }
        }

        return C;
    }

    public static void printMatrix(int[][] M) {
	int inf = Integer.MAX_VALUE;
	for (int i = 0; i < M.length; i++) {
	    for (int j = 0; j < M[0].length; j++)
		if (M[i][j] == inf)
		    System.out.print("inf ");
		else
		    System.out.print(String.format("%3d ",M[i][j]));
            System.out.println();
        }
	System.out.println();
    }

    public static void main(String[] args) {
	Random ran = new Random();
	long t1, t2;
	int size   = 300;
	int [][] W = new int[size][size];
	
	for (int i = 0; i < W.length; i++) {
	    for (int j = i; j < W[0].length; j++) {
		int r =ran.nextInt(10);
		W[i][j] = r;
		W[j][i] = r;
	    }
	}

	// make a copy of the edge weight matrix W
	int[][] R = W.clone();

	System.out.println(String.format("Comparing runtimes for distance matrix computation for matrices of size %d x %d",size,size));
	    
	// find the distance matrix by (n-2) subsequent matrix matrix multiplications
	// R = (((W*W)*W)*...*W) = W^(n-1)
	t1 = System.currentTimeMillis();
	for (int i = 0 ; i < size - 2; i++)
	    R = multiplyModSquared(R,W);
	t2 = System.currentTimeMillis();
	
	System.out.println(String.format("The n-2 multiplications took %3.2f seconds",((t2-t1)/1000.0)));

	// set the R=W (re-initialize)
	R = W.clone();

	// find the distance matrix by ceil(log_2(n-1)) subsequent matrix matrix
	// mulitplications via repeated squaring R = (((W^2)^2)^2...)^2
	t1 = System.currentTimeMillis();
	for (int i=0; i < Math.ceil( Math.log(size-1) / Math.log(2) ); i++)
	    R = multiplyModSquared(R,R);
	t2 = System.currentTimeMillis();

	System.out.println(String.format("The ceil(log_2(n-1)) multiplications took %3.2f seconds",((t2-t1)/1000.0)));

    }
}
